#Standard format of AI/player input processing module - every function that is here has to remain here with the same arguments
#Otherwise the RTS won't work

#Imports locals from pygame so input can be processed
from pygame.locals import *
import pygame

from collections import deque
from collections import Counter #Used to find most common items in list Counter(list).most_common()

import random #Used to make units not pathfind at the same time

#Helper modules
from . import pathfinding_module

class Controller():
    def __init__(self,mapa,team_manager,kamera,tile_size,screen):
        self.mapa = mapa
        self.team_manager = team_manager
        self.kamera = kamera
        self.tile_size = tile_size
        self.screen = screen

        self.entities = []

        self.selected = [] #Stores units and buildings that are selected
        self.selected_production_majority = "" #Stores which type of entity that can produce other entities is most commonly selected
 
        self.current_production_possibilities = [] #Stores what production target can be chosen from
        self.current_production_choice = 0 #Stores current choice from the production posssibilities

        for unit in self.team_manager.unitManager.all_units:
            self.entities.append(Entity_host(self,unit))
        for building in self.team_manager.buildingManager.all_buildings:
            self.entities.append(Entity_host(self,building))

        self.restricted_positions = []
        self.restricted_positions = self.team_manager.blocked_positions

        self.construct_building = False #@todo document
    
    def controller_cycle(self,user_input):
        #Calculate the selected production majority and production possibilities
        names = []
        name_to_production_possibilities = {}
        for entity in self.selected:
            if entity.can_construct:
                name_to_production_possibilities[entity.entity.name] = entity.can_construct
                names.append(entity.entity.name)
                
        if names:
            self.selected_production_majority = Counter(names).most_common(1)[0][0]
            self.current_production_possibilities = name_to_production_possibilities[self.selected_production_majority]
            if self.current_production_choice > len(self.current_production_possibilities)-1: #Production choice isn't in the possibilities
                self.current_production_choice = 0
        else: #No production entity is selected
            self.selected_production_majority = ""
            self.current_production_possibilities = []
            self.current_production_choice = 0

        #Finds new entities in team manager that has no entity_hosts and gives them to entity_host
        for unit in self.team_manager.unitManager.all_units:
            try:
                tmp = unit.entity_host_mark
            except AttributeError:
                self.entities.append(Entity_host(self,unit))
        for building in self.team_manager.buildingManager.all_buildings:
            try:
                tmp = building.entity_host_mark
            except AttributeError:
                self.entities.append(Entity_host(self,building))
        
        self.restricted_positions = self.team_manager.blocked_positions
        order_string = ""
        order_string += self.process_input(user_input)
        order_string += self.update()
        return order_string #Has to return valid order string ("" is valid too)

    def process_input(self,user_input):
        output = ""
        for event, mouse_position in user_input:
            mouse_x = int((mouse_position[0] + self.kamera.x)/self.tile_size)
            mouse_y = int((mouse_position[1] + self.kamera.y)/self.tile_size)

            if event.type == MOUSEBUTTONDOWN:
                mouse_button = event.button
                keyboard_mods = pygame.key.get_mods() #keyboard_modifiers

                if mouse_button == 1: #Left click
                    self.construct_building = False #@todo document
                    #Selects entity
                    if not keyboard_mods & KMOD_SHIFT:
                        self.selected = [] #Deselects all entities if shift is not held
                    selected_entity = self.detect_entity_using_point(mouse_x,mouse_y)
                    if selected_entity:
                        self.selected.append(selected_entity)

                if mouse_button == 3 and self.selected: #Right click
                    if self.construct_building and not (mouse_x,mouse_y) in self.restricted_positions:
                        build_position = (mouse_x,mouse_y)
                        entity_used_to_build = ""
                        for entity in self.entities:
                            if entity.entity.name == self.selected_production_majority:
                                if abs(build_position[0]-entity.entity.x)+abs(build_position[1]-entity.entity.y) == 1:
                                    entity_used_to_build = entity
                                    break

                        if entity_used_to_build:
                            construction_x, construction_y = build_position
                            output += ("u:"+str(entity.entity.id)+":bu:"+str(construction_x)+":"+str(construction_y)+":"
                                           +self.current_production_possibilities[self.current_production_choice]+":"+
                                           str(entity.how_construct[self.current_production_possibilities[self.current_production_choice]])+
                                           ";")

                        return output
                            
                        
                    for entity in self.selected: #Order all selected entities to move or attack or work or mine on the clicked position
                        #Reset variables of the entities
                        entity.target = False
                        entity.replan_route = False
                        entity.target_entity = False
                        entity.mine_ore_at = False
                        entity.ore_type = False
                        entity.return_phase = False
                        entity.drop_off_entity = False
                        
                        if (mouse_x,mouse_y) in self.restricted_positions.values(): #Attack, follow or work
                            #Targets entity at clicked position
                            entity.target = (mouse_x,mouse_y)
                            entity.replan_route = True
                            entity.target_entity = False
                            for key_entity, position in self.restricted_positions.items():
                                if position == entity.target:
                                    entity.target_entity = key_entity
                                    break
                                
                        elif entity.miner_ai and (mouse_x,mouse_y) in self.team_manager.parent.ore_manager.ores: #Mine
                            if (entity.entity.tool[0] in self.team_manager.parent.ore_manager.ore_types[
                                self.team_manager.parent.ore_manager.ores[(mouse_x,mouse_y)][0]]):
                                    entity.mine_ore_at = (mouse_x,mouse_y)
                                    entity.ore_type = self.team_manager.parent.ore_manager.ores[(mouse_x,mouse_y)][0]
                                    entity.target = (mouse_x,mouse_y)
                                    entity.replan_route = True
                                    
                        else: #Move
                            entity.target = (mouse_x,mouse_y)
                            entity.target_entity = False
                            entity.replan_route = True

                #Mouse wheel up
                if mouse_button == 4:
                    self.current_production_choice += 1
                    if self.current_production_choice > len(self.current_production_possibilities)-1: #Production choice is too large, changes to zero
                        self.current_production_choice = 0

                #Mouse wheel down
                if mouse_button == 5:
                    self.current_production_choice -= 1
                    if self.current_production_choice < 0:  #Production choice is too small, changes to max
                        self.current_production_choice = len(self.current_production_possibilities)-1
                        
            if event.type == KEYDOWN:
                if event.key == K_RETURN: #Used to start production of unit/building
                    self.construct_building = False
                    for entity in self.selected: 
                        if entity.entity.name == self.selected_production_majority:
                            if entity.entity_type == "unit":
                                self.construct_building = True
                                break
                            elif entity.entity.time_to_finish_building == -1: #Buildings producing units
                                construction_positions = [[0,-1],[1,0],[0,1],[-1,0]]
                                construction_x, construction_y = [-100,-100]
                                for possible_construction_position in construction_positions:
                                    if (not((entity.entity.x+possible_construction_position[0],entity.entity.y+possible_construction_position[1])
                                            in self.restricted_positions.values())):
                                        if (self.mapa[entity.entity.x+possible_construction_position[0]]
                                            [entity.entity.y+possible_construction_position[1]]):
                                            construction_x, construction_y = (entity.entity.x+possible_construction_position[0],
                                                                          entity.entity.y+possible_construction_position[1])
                                            break

                                        
                                output += ("b:"+str(entity.entity.id)+":bu:"+str(construction_x)+":"+str(construction_y)+":"
                                           +self.current_production_possibilities[self.current_production_choice]+":"+
                                           str(entity.how_construct[self.current_production_possibilities[self.current_production_choice]])+
                                           ";")
                                break

        return output
                    

    def update(self):
        #Removes destroyed units and buildings
        for i in range(len(self.entities)-1,-1,-1):
            if self.entities[i].entity.hp <= 0:
                self.entities.pop(i)

        order_string = ""

        for entity in self.entities:
            order_string += entity.update()

        return order_string

    def detect_entity_using_point(self,x,y):
        for entity in self.entities:
            if entity.entity.x == x and entity.entity.y == y:
                return entity

    def draw_cycle(self):
        for entity in self.selected: #Draws green squares around all selected entities
            pygame.draw.rect(self.screen,[0,255,0],[entity.entity.x*self.tile_size-self.kamera.x,entity.entity.y*self.tile_size-self.kamera.y,
                                                    self.tile_size,self.tile_size],2)

        #GUI
        #Draws squares for entity images
        pygame.draw.rect(self.screen,[0,255,0],[self.screen.get_size()[0]/40*37.5-2,1*self.tile_size-2,
                                                self.tile_size+2,self.tile_size+2],2) #Selected production majority
        pygame.draw.rect(self.screen,[0,0,255],[self.screen.get_size()[0]/40*37.5-2,2.5*self.tile_size-2,
                                                self.tile_size+2,self.tile_size+2],2) #Selected production target
        #Draws entity images into squares
        if self.selected_production_majority: #Selected production majority image will be drawn
            self.screen.blit(self.team_manager.drawing_engine.images[self.selected_production_majority][0],
                             (self.screen.get_size()[0]/40*37.5,1*self.tile_size))
            
        if self.current_production_possibilities: #Selected production target will be drawn
            self.screen.blit(self.team_manager.drawing_engine.images[self.current_production_possibilities[self.current_production_choice]][0],
                             (self.screen.get_size()[0]/40*37.5,2.5*self.tile_size))
        

class Entity_host(): #Contains unit/building
    def __init__(self,parent,entity):
        self.parent = parent
        self.entity = entity
        self.entity_type = "unit" #Stores entity type - will be used till unification comes (unification of unit and building class in game source code)
        if str(type(self.entity)) == "<class 'moduleBuilding.BasicBuilding'>":
            self.entity_type = "building"

        self.target = (self.entity.x,self.entity.y) #Target position for entity to move to
        self.target_entity = False #Entity that should be attacked or worked at
        
        self.replan_route = False
        self.route = deque()

        #Mining
        self.miner_ai = False #Changes behaviour of the entity if the entity has mining tool
        if self.entity.tool[0] != "no":
            self.miner_ai = True

        self.mine_ore_at = False #Stores position of ore that should be mined
        self.ore_type = False
        self.return_phase = False #Indicates if the entity wants to drop off mined resources

        #Resource processing
        self.can_process = [] #Stores materials that can be processed by this entity
        self.processing_recipes = [] #Stores [output_resource, recipe_number]
        for output_resource, recipes in self.parent.team_manager.recipe_manager.resource_recipes.items():
            for recipe_number in range(len(recipes)):
                if self.entity.tool[3] in recipes[recipe_number][0]: #Checks if the processing tool is usable for the recipe
                    self.processing_recipes.append([output_resource,recipe_number])
                    for used_resource in recipes[recipe_number][1]:
                        self.can_process.append(used_resource)

        #Construction
        self.can_construct = [] #Stores only units or buildings that can be produced by this entity
        self.how_construct = {} #Stores units or builiding that can be constructed by this entity paired with recipe number
        for output_entity, recipes in self.parent.team_manager.recipe_manager.recipes.items():
            for i in range(len(recipes)):
                if self.entity.tool[1] in recipes[i][0]:
                    self.can_construct.append(output_entity)
                    self.how_construct[output_entity] = i

        #Puts entity host mark on the entity so it is possible to know if it is controlled by any entity host
        self.entity.entity_host_mark = True
                    
    def update(self):
        orders = ""

        #Mining phase
        if self.mine_ore_at and not self.return_phase:
            #Mining itself
            if (self.entity.cargo_capacity > self.entity.cargo_filled and
                (abs(self.mine_ore_at[0]-self.entity.x) + abs(self.mine_ore_at[1]-self.entity.y)) <= 1): #Entity is close enough to ore to mine
                orders += self.entity_type[0]+":"+str(self.entity.id)+":mi:"+str(self.mine_ore_at[0])+":"+str(self.mine_ore_at[1])+";"

            #Checks if the ore has been mined out, if yes, find another ore to mine
            if not self.mine_ore_at in self.parent.team_manager.parent.ore_manager.ores:
                self.mine_ore_at = False
                lowest_distance = -1
                for position, ore_properties in self.parent.team_manager.parent.ore_manager.ores.items(): #Chooses new closest ore of the same type
                    if self.ore_type == ore_properties[0] and ore_properties[1] > 0:
                        if lowest_distance == -1 or (abs(position[0]-self.entity.x) + abs(position[1]-self.entity.y)) < lowest_distance:
                            lowest_distance = (abs(position[0]-self.entity.x) + abs(position[1]-self.entity.y))
                            self.mine_ore_at = position
                            self.target = self.mine_ore_at
                            self.replan_route = True

            if (self.entity.cargo_capacity <= self.entity.cargo_filled and not self.return_phase) or not self.mine_ore_at:
                #Cargo capacity has been filled, entity should drop off the cargo
                #Chooses place to drop off the cargo
                lowest_distance = -1
                self.drop_off_entity = False
                for potential_drop_off_entity in self.parent.entities:
                    if self.ore_type in potential_drop_off_entity.can_process:
                        if potential_drop_off_entity.entity.cargo_capacity > potential_drop_off_entity.entity.cargo_filled: #Checks if the place has enough capacity
                            if (lowest_distance == -1 or (abs(potential_drop_off_entity.entity.x-self.entity.x) +
                                                          abs(potential_drop_off_entity.entity.y-self.entity.y)) < lowest_distance):
                                lowest_distance =(abs(potential_drop_off_entity.entity.x-self.entity.x) +
                                                  abs(potential_drop_off_entity.entity.y-self.entity.y))
                                self.drop_off_entity = potential_drop_off_entity

                if self.drop_off_entity:
                    self.target = (self.drop_off_entity.entity.x,self.drop_off_entity.entity.y)
                    self.replan_route = True
                    self.return_phase = True

        #Going to unload the mined ore
        if self.return_phase:
            #Drop off entity moves, the route is replanned
            if self.drop_off_entity and (self.drop_off_entity.entity.x != self.target[0] or self.drop_off_entity.entity.y != self.target[1]):
                self.target = (self.drop_off_entity.entity.x,self.drop_off_entity.entity.y)
                self.replan_route = True

            #Unloads the cargo if the drop off entity is close enough and has enough capacity
            if ((abs(self.drop_off_entity.entity.x-self.entity.x) + abs(self.drop_off_entity.entity.y-self.entity.y)) <= 1
                and self.drop_off_entity.entity.cargo_capacity > self.drop_off_entity.entity.cargo_filled):
                orders += (self.entity_type[0]+":"+str(self.entity.id)+":un:"+str(self.drop_off_entity.entity.x)+":"+str(self.drop_off_entity.entity.y)
                           +":"+str(self.ore_type)+":"+str(self.entity.cargo[self.ore_type])+";")

            #Ends the return phase once all the ore cargo is unloaded
            if self.entity.cargo[self.ore_type] == 0:
                self.return_phase = False
                if self.mine_ore_at:
                    self.target = self.mine_ore_at
                else:
                    self.target = (self.entity.x,self.entity.y)
                    self.ore_type = False
                self.replan_route = True
        #End of mining phase

        #Processing phase
        if self.processing_recipes: #Checks if the entity can process anything
            if self.entity.time_to_finish_processing == -1: #Checks if the entity isn't processing something currently
                for recipe_output, recipe_number in self.processing_recipes: #Iterates through all possible recipes
                    requirements = self.parent.team_manager.recipe_manager.resource_recipes[recipe_output][recipe_number]
                    do_the_recipe = True
                    for i in range(len(requirements[1])): #Iterates over resource requirements for the recipe
                        if not self.entity.cargo[requirements[1][i]] >= requirements[2][i]: #Checks if the entity has enough resources
                            do_the_recipe = False #Entity didn't have enough resources

                    if do_the_recipe: #Recipe is possible to do, so it will be done
                        orders += self.entity_type[0]+":"+str(self.entity.id)+":pr:"+str(recipe_output)+":"+str(recipe_number)+";"
                
        #End of processing phase
                                            
        #Attack phase
        #Attack target checking
        if self.target_entity and self.target_entity.hp <= 0: #Removes reference to the target entity if has been destroyed
            self.target_entity = False

        #If the target entity moves, the route is replanned
        if self.target_entity and (self.target_entity.x != self.target[0] or self.target_entity.y != self.target[1]):
            self.target = (self.target_entity.x,self.target_entity.y)
            self.replan_route = True
        
        #Attacking
        if not self.entity.weapon == (0,0,0): #Weapon "no", unusable weapon
            #Weapons are tuples (damage,range,slowness)
            if self.entity.reload >= self.entity.weapon[2]: #Checks if the entity has enough of "attack_charge" to overcome slowness of the weapon
                if (self.target_entity and self.target_entity.parent.parent.team_number != self.parent.team_manager.team_number and
                    (abs(self.target_entity.x-self.entity.x) + abs(self.target_entity.y-self.entity.y)) <= self.entity.weapon[1]):
                    orders += self.entity_type[0]+":"+str(self.entity.id)+":fi:"+str(self.target_entity.x)+":"+str(self.target_entity.y)+";"
                    
                else: #Attacks enemy unit with highest score (score is calculated by dividing 1000 by entity hp and entity distance
                    entity_to_attack = False
                    target_score = 0
                    for key_entity, position in self.parent.restricted_positions.items():
                        if (key_entity.parent.parent.team_number != self.parent.team_manager.team_number and
                            (abs(position[0]-self.entity.x)+abs(position[1]-self.entity.y)) <= self.entity.weapon[1]):
                            score = 1000/(key_entity.hp+1)/((abs(position[0]-self.entity.x)+abs(position[1]-self.entity.y))+1)
                            if score > target_score:
                                target_score = score
                                entity_to_attack = key_entity
                                
                    if entity_to_attack:
                        orders += self.entity_type[0]+":"+str(self.entity.id)+":fi:"+str(entity_to_attack.x)+":"+str(entity_to_attack.y)+";"
        #End of attack phase

        #Working on buildings using units #@todo finish
        if (self.target_entity and str(type(self.target_entity)) == "<class 'moduleBuilding.BasicBuilding'>" and self.target_entity.time_to_finish > 0 and
            self.target_entity.tool_to_build == self.entity.tool[1]):
            if abs(self.target_entity.x-self.entity.x) + abs(self.target_entity.y-self.entity.y) <= 1:
                orders += self.entity_type[0]+":"+str(self.entity.id)+":wo:"+str(self.target_entity.x)+":"+str(self.target_entity.y)+";"
                            
        #Replanning route
        if self.replan_route and self.entity_type == "unit":
            if random.randint(0,1) == 1:
                self.route = pathfinding_module.find_path(self.parent.mapa,(self.entity.x,self.entity.y),self.target,self.parent.restricted_positions.values())
                self.replan_route = False
                self.route.popleft() #Removes start of route because it has the same position as the current position

        #Moving
        if self.entity_type == "unit": #Unit moving
            #Entity follow target entity at the max. range to prevent blocking or getting too close
            if not self.target_entity or not ((abs(self.target_entity.x-self.entity.x) + abs(self.target_entity.y-self.entity.y)) <= self.entity.weapon[1]):
                if (not self.mine_ore_at or self.return_phase
                    or not (abs(self.mine_ore_at[0]-self.entity.x) + abs(self.mine_ore_at[1]-self.entity.y)) <= 1):
                    if self.route:
                        if self.entity.move_charge >= self.entity.slowness:
                            if not self.route[0] in self.parent.restricted_positions.values():
                                next_position = self.route.popleft()
                                direction = (next_position[0]-self.entity.x,next_position[1]-self.entity.y)
                                orders += "u:"+str(self.entity.id)+":mo:"+str(direction[0])+":"+str(direction[1])+";"
                            else:
                                self.replan_route = True
                
        return orders





