#Client file
#Import and setting up drivers
import pygame
from pygame.locals import *

#Imports modules
import simple_modules.moduleFilePathFormer #Creates paths to files
import simple_modules.moduleGameClock #Regulates game speed and makes logic frame independent from rendering frame
import simple_modules.moduleCamera #Moves shown screen

import moduleNetwork #Network library

import moduleDrawingEngine

import moduleMapLoader #Loads maps

import moduleTeamsManager #Teams manager, used for top level managment

debug_mode = False #Removes user input when True

def main_program():
    pygame.init()

    #Color definition, will be probably removed somewhere in the future #@todo remove when no longer necessary
    black    = (   0,   0,   0)
    white    = ( 255, 255, 255)
    green    = (   0, 255,   0)
    red      = ( 255,   0,   0)
    blue     = (   0,   0, 255)
    violet   = ( 128,   0, 127)
    yellow   = ( 255, 255,   0)
    travnik  = (   0, 214,   0)
    hneda    = ( 166, 124,  35)
    voda     = (  20, 154, 181)

    #FilePathFormer initialization
    path_creator = simple_modules.moduleFilePathFormer.FilePathFormer("Game_folder")

    tile_size = 40 #@todo place where appropriate

    screen_size = [400,400]
    screen = pygame.display.set_mode(screen_size,RESIZABLE)

    screen_size[0] = screen.get_size()[0]
    screen_size[1] = screen.get_size()[1]

    pygame.display.set_caption("PyRTS client")
    screen.fill(black)
    pygame.display.update()
    cycle = True

    #Initalizes game clock
    game_clock = simple_modules.moduleGameClock.GameClock(200)#20)

    #Camera, allows moving view field
    kamera = simple_modules.moduleCamera.Camera(0,0,screen_size) #must start at zero

    #User input
    if debug_mode:
        server_adress = moduleNetwork.get_adress()
    else:
        server_adress = input("Enter server adress: ")
    
    #Connects to the server
    client_socket = moduleNetwork.ClientSocket(8192)
    client_socket.connect(server_adress)
    #Receives data required for game initialization and map loading
    team_number = int(client_socket.receive()) #This client team number
    team_count = int(client_socket.receive()) #Amount of teams total
    map_path = (client_socket.receive()).split(":") #Path to map folder from the folder Maps in Game_folder

    #Loads map files
    map_path = path_creator.create_path_to_file(["Maps"]+map_path) #Finalizes the map path so it will be usable
    terrain_map, unit_file, building_file, ore_file = moduleMapLoader.load_map(map_path) #Uses the path to load the files
    #unit_file, building_file and ore_file have to be run through the map loader again with teams_manager because they are used to write data into it

    #Initializes drawing engine
    drawing_engine = moduleDrawingEngine.DrawingEngine(kamera,screen,tile_size,team_count,terrain_map)
    
    #Creates TeamsManager, to manage nearly everything #@todo rename to GameManager maybe?
    teams_manager = moduleTeamsManager.TeamsManager(team_count,terrain_map,drawing_engine)

    #Writes data from unit_file, building_file and ore_file into teams_manager
    moduleMapLoader.finalize_loading(unit_file,building_file,ore_file,teams_manager)

    #Imports and initializes controller
    from Slot import controller
    team_controller = controller.Controller(terrain_map,teams_manager.teamManagers[team_number],kamera,tile_size,screen)

    #Temporary low FPS fix for multiclient play - circumvents the GameClock
    temporary_fps_fix = True #@todo remove and fix the fps ASAP

    while cycle == True:
        required_logic_cycles = game_clock.update() #Checks how many logic cycles should happen during this frame
        if temporary_fps_fix:
            required_logic_cycles = 1
        for i in range(required_logic_cycles):
            ###############################
            # Basic user input processing #
            ###############################
            #@todo probably move somewhere else
            
            event_list = []
            
            for event in pygame.event.get():
                if event.type == QUIT:
                    cycle = False
                if event.type == VIDEORESIZE: #Gets the event when user resizes the window
                    #Resizes the window
                    screen_size[0] = event.w
                    screen_size[1] = event.h
                    screen = pygame.display.set_mode(screen_size,RESIZABLE)
                if event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        cycle = False
                    if event.key == K_SPACE:
                        #Writes FPS
                        print(game_clock.get_fps())
                    else:
                        event_list.append((event,pygame.mouse.get_pos()))
                else:
                    event_list.append((event,pygame.mouse.get_pos()))
                    
            #Checks which directional keys (arrows) are held down
            directional_keys_held_down = pygame.key.get_pressed()            
            #Those conditionals then move camera (viewpoint) according to held keys
            if directional_keys_held_down[K_RIGHT]:
                kamera.x += 10
            if directional_keys_held_down[K_LEFT]:
                kamera.x -= 10
            if directional_keys_held_down[K_DOWN]:
                kamera.y += 10
            if directional_keys_held_down[K_UP]:
                kamera.y -= 10

            ##################################
            # End of direct input processing #
            ##################################

            #Gives player input to controller instance
            controller_output = team_controller.controller_cycle(event_list) #Controller should then return orders for its team

            #Informs server about ending the game and ends the game
            if cycle == False:
                client_socket.send("disconnect")
                client_socket.socket.close()
                pygame.quit()
                input("Press ENTER to end the program")
                return

            #Sends input from controller to server and then receives input from all controllers
            client_socket.send(controller_output)
            all_controllers_input = client_socket.receive().split("?:?") #Receives message from the server (separator in it is ?:?)
            
            #Losing and winning
            if all_controllers_input[0] == "You have been defeated":
                client_socket.socket.close()
                print(all_controllers_input[0])
                cycle = False
                input("Press ENTER to end the program")
                pygame.quit()
                return

            if all_controllers_input[0] == "You are victorious":
                client_socket.socket.close()
                print(all_controllers_input[0])
                cycle = False
                input("Press ENTER to end the program")
                pygame.quit()
                return
            
            #If the all_controllers_input is too short, it is artificially prolonged to not cause errors #@toto replace with something better
            while len(all_controllers_input) < team_count:
                all_controllers_input.append("")

            teams_manager.give_orders_to_teams(all_controllers_input) #Gives the received controllers input to team managers
                
            teams_manager.update() #Everything updates
            
            #######################################################################################################
            ############################### End of logic cycle ####################################################
            #######################################################################################################

        #Must be always called before standard drawing - clears the screen
        screen.fill(black) #Used so nothing will be drawn outside of the playing field

        #Draws terrain
        drawing_engine.draw_terrain()

        #All units and buildings are drawn on the screen
        teams_manager.draw_all()

        #Draws GUI side bar on the right side of screen
        drawing_engine.draw_side_bar()

        #Calls controller drawing cycle
        team_controller.draw_cycle()

        #Shows the drawn objects
        pygame.display.flip()

        #Tells the game clock that one frame has passed
        if temporary_fps_fix:
            game_clock.tick(10)
        else:
            game_clock.tick(60) #Locks at 60 FPS to prevent extensive usage of CPU/GPU

    pygame.quit()


if __name__ == "__main__": #Prevents recursion with multiprocessing and prevents execution when imported
    main_program()


