#Network library

import socket as socket_lib

#Sub-level function for receiving
def receive_all(socket,length): #Function that receives all data of set length from socket
    message = ""
    while not len(message) == length:
        message += socket.recv(length-len(message)).decode()
    return message

def receive(socket):
    if not socket: #Checks if the socket is socket
        return
    
    message = ""
    length_indicator = receive_all(socket,6) #Receives message indicating length of real message
    if not length_indicator.split() == []:
        length_indicator = int(length_indicator.split()[0]) #Remove " " from length indicator and converts to int
        message = receive_all(socket,length_indicator)
        return message #Returns received data

def send(socket,message):
    if not socket: #Checks if the socket is socket
        return
    
    length_indicator = str(len(message)) #Creates length indicator
    
    if len(length_indicator) > 6: #If message is longer than 999999 characters, return error
        print (message)
        raise ValueError("Message is too long, max length is 999999 characters")
    
    while len(length_indicator) < 6: #If message has less than 100000 characters, prolongs the indicator to have fixed length
        length_indicator = " " + length_indicator
        
    socket.sendall(length_indicator.encode()) #Sends message indicator
    socket.sendall(message.encode()) #Sends message

def get_adress():
    return socket_lib.gethostname()

def get_ip_adress():
    return socket_lib.gethostbyname(socket_lib.gethostname())

class ClientSocket():
    def __init__(self,port):
        self.socket = socket_lib.socket()
        self.socket.setsockopt(socket_lib.SOL_SOCKET, socket_lib.SO_REUSEADDR,1)
        self.socket.setsockopt(socket_lib.IPPROTO_TCP, socket_lib.TCP_NODELAY, 1)
        self.socket.settimeout(None)
        
        self.port = port
        
    def connect(self,host): #Connects to host
        self.socket.connect((host,self.port))

    def send(self,message):
        send(self.socket,message)

    def receive(self):
        return receive(self.socket)
        

class ServerSocket():
    def __init__(self,port):
        self.socket = socket_lib.socket()
        self.socket.setsockopt(socket_lib.SOL_SOCKET, socket_lib.SO_REUSEADDR,1)
        self.socket.setsockopt(socket_lib.IPPROTO_TCP, socket_lib.TCP_NODELAY, 1)
        self.socket.settimeout(None)
        
        self.port = port

        self.connections = []
        self.adresses = []

        self.start()
    
    def start(self): #Binds socket to public IP
        self.socket.bind(("0.0.0.0",self.port))
        self.socket.listen(5)
        
    def accept(self): #Accepts connection
        connection, adress = self.socket.accept()
        self.connections.append(connection)
        self.connections[-1].settimeout(None)
        self.adresses.append(adress)
        return connection

    def receive_from_all(self):
        output = []
        for i in self.connections:
            output.append(receive(i))

        return output

    def send_everyone(self,message):
        for i in self.connections:
            send(i,message)





        
