import moduleUnit
import moduleBuilding
import moduleResourceManager

def merge_dicts(*dicts):
    result = {}
    for dictionary in dicts:
        result.update(dictionary)
    return result

class TeamManager(): #Top manager of team, process controller input and contains building and unit managers
    def __init__(self, parent, team_number, mapa, drawing_engine):
        self.parent = parent
        self.team_number = team_number
        self.UBWTTypes = self.parent.UBWTTypes
        self.recipe_manager = self.parent.recipe_manager
        self.drawing_engine = drawing_engine
        
        self.unitManager = moduleUnit.SimpleUnitManager(self, mapa, self.drawing_engine)
        self.buildingManager = moduleBuilding.BuildingManager(self, mapa, self.drawing_engine)
        self.resourceManager = moduleResourceManager.ResourceManager()

        self.blocked_positions = [] #Stores positions blocked by all buildings and units (except units of this team

    def update(self,unit_blocked_positions,building_blocked_positions):
        self.blocked_positions = merge_dicts(*(unit_blocked_positions+building_blocked_positions))

        blocking_positions = [self.unitManager.update(),self.buildingManager.update()]

        return blocking_positions

    def draw_all(self):
        self.unitManager.draw_all_units()
        self.buildingManager.draw_all_buildings()

    def detect_entity_using_position(self,x,y):
        entity = self.unitManager.detect_unit_using_position(x,y)
        unitORbuilding = "unit"
        if not entity:
            entity = self.buildingManager.detect_building_using_position(x,y)
            unitORbuilding = "building"

        if not entity:
            return False

        return (entity,unitORbuilding)

    def create_unit(self, x, y, unit_type):
        self.unitManager.create_unit(x, y, unit_type)

    def create_building(self, x, y, building_type, time_to_finish = 0, tool_to_build = ""):
        self.buildingManager.create_building(x, y, building_type, time_to_finish, tool_to_build)

    def process_orders(self,orders): #Processes order string (not player input (player input is processed in controller), but orders for exact buildings and units)
        #Order has to be in format b:187:mo:1:0; - first is building(b) or unit(u), second is ID, then comes the order name (mo-move),(fi-fire)
        #then arguments for order and last comes order separator (;)
        if orders: #Checks if the orders aren't empty
            orders = orders.split(";")
            for order in orders:
                if order: #Checks if the order isn't empty
                    order = order.split(":")
                    
                    entity = "" #Entity that should do something
                    if order[0] == "b":
                        entity = self.buildingManager.id_system[int(order[1])]
                    elif order[0] == "u":
                        entity = self.unitManager.id_system[int(order[1])]
                        
                    if order[2] == "mo": #Entity move() function
                        entity.move((int(order[3]),int(order[4])))
                    elif order[2] == "fi": #Entity fire() function
                        entity.fire((int(order[3]),int(order[4])))
                    elif order[2] == "wo": #Entity work_on_building() function
                        entity.work_on_building((int(order[3]),int(order[4])))
                    elif order[2] == "mi": #Entity mine() function
                        entity.mine((int(order[3]),int(order[4])))
                    elif order[2] == "un": #Entity unload_cargo() function
                        entity.unload_cargo((int(order[3]),int(order[4])),order[5],int(order[6]))
                    elif order[2] == "bu": #Entity build() function
                        entity.build((int(order[3]),int(order[4])),order[5],int(order[6]))
                    elif order[2] == "pr": #Entity process_resource() function
                        entity.process_resource(order[3],int(order[4]))







