import moduleTeamManager
import moduleUBWTTypeManager
import moduleRecipeManager
import moduleOreManager

class TeamsManager(): #Stores all team managers
    def __init__(self, amount_of_teams, mapa, drawing_engine):
        self.teamManagers = []
        self.drawing_engine = drawing_engine
        
        self.UBWTTypes = moduleUBWTTypeManager.UBWTTypeManager()
        self.recipe_manager = moduleRecipeManager.RecipeManager()
        self.ore_manager = moduleOreManager.OreManager(self.drawing_engine)
        
        self.unit_blocked_positions = [] #Contains all positions blocked by units, with one list for each team
        self.building_blocked_positions = [] #Contains all positions blocked by buildings, with one list for each team
        
        team_number = 0
        for i in range(amount_of_teams):
            self.teamManagers.append(moduleTeamManager.TeamManager(self, team_number, mapa, self.drawing_engine))
            team_number += 1
            self.unit_blocked_positions.append({})
            self.building_blocked_positions.append({})

    def update(self):
        for i in range(len(self.teamManagers)):
            #Updates blocked positions and updates team manager using them
            self.unit_blocked_positions[i], self.building_blocked_positions[i] = self.teamManagers[i].update(self.unit_blocked_positions,
                                                                                                              self.building_blocked_positions)

        self.ore_manager.update()

    def draw_all(self):
        self.ore_manager.draw_all()
        for i in self.teamManagers:
            i.draw_all()

    def create_unit(self, team, x, y, unit_type):
        self.teamManagers[team].create_unit(x, y, unit_type)

    def create_building(self, team, x, y, building_type):
        self.teamManagers[team].create_building(x, y, building_type)

    def give_orders_to_teams(self,orders): #Takes list with orders for each time and gives every team manager corresponding orders
        while len(orders) < len(self.teamManagers):
            orders.append("")
        for i in range(len(self.teamManagers)):
            self.teamManagers[i].process_orders(orders[i])
