#Module for loading and storing weapon, tool, unit and building types - for example: unit "tank" - [hp,weapon,tool]

from simple_modules import moduleFilePathFormer #Necessary to access files with unit and building types in other directories

class UBWTTypeManager():
    def __init__(self):
        self.unit_types = {}
        self.building_types = {}
        self.weapon_types = {"no":(0,0,0)} #Contains unusable weapon "no" by default. This weapon is used for unarmed entities
        self.tool_types = {"no":("no","no","no","no")} #Contains unusable tool "no" by default. This tool is used for entities that can't mine/repair/build/process

        self.filePathFormer = moduleFilePathFormer.FilePathFormer("Game_folder")

        self.load_unit_types()
        self.load_building_types()
        self.load_weapon_types()
        self.load_tool_types()

    def load_unit_types(self): #Loads unit types from UBWTypes\unit.txt
        file = open(self.filePathFormer.create_path_to_file(["UBWTTypes","unit.txt"]),"r")
        for line in file:
            if not line[0] == "#":
                if "\n" in line: #Removes line ending
                    line = line[:-1]
                line = line.split(":")
                #Loads hp, weapon and tool, cargo_capacity, slowness
                self.unit_types[line[0]] = tuple([float(line[1]),line[2],line[3],int(line[4]),int(line[5])])

        file.close()

    def load_building_types(self): #Loads building types from UBWTypes\building.txt
        file = open(self.filePathFormer.create_path_to_file(["UBWTTypes","building.txt"]),"r")
        for line in file:
            if not line[0] == "#":
                if "\n" in line: #Removes line ending
                    line = line[:-1]
                line = line.split(":")
                #Loads hp, weapon and tool, cargo_capacity
                self.building_types[line[0]] = tuple([float(line[1]),line[2],line[3],int(line[4])])

        file.close()

    def load_weapon_types(self): #Loads weapon types from UBWTypes\weapon.txt
        file = open(self.filePathFormer.create_path_to_file(["UBWTTypes","weapon.txt"]),"r")
        for line in file:
            if not line[0] == "#":
                if "\n" in line: #Removes line ending
                    line = line[:-1]
                line = line.split(":")
                #Loads attack damage, attack range and attack slowness
                self.weapon_types[line[0]] = tuple([float(line[1]),float(line[2]),float(line[3])])

        file.close()

    def load_tool_types(self): #Loads tool types from UBWTTypes\tool.txt #WIP #@todo improve and finish tool system
        file = open(self.filePathFormer.create_path_to_file(["UBWTTypes","tool.txt"]),"r")
        for line in file:
            if not line[0] == "#":
                if "\n" in line: #Removes line ending
                    line = line[:-1]
                line = line.split(":")
                #Loads abilities of the tool (mining, building, repairing, resource processing)
                self.tool_types[line[0]] = tuple([line[1],line[2],line[3],line[4]])

        file.close()
