#Module with basic unit and unit manager classes

import math #Used for rounding up using ceil()
from collections import defaultdict #dictionary that returns default value when the value isn't found

class BasicUnit():
    def __init__(self, parent, name, x, y, hp, weapon, tool, cargo_capacity, slowness):
        self.parent = parent
        self.name = name
        self.id = -1 #id is -1 until it is changed to real id
        
        self.x = x
        self.y = y
        self.size = [1,1]
        
        self.hp = hp #Hitpoints
        self.weapon = weapon #Weapon - format (damage, range, speed/slowness)
        self.tool = tool #Tool - mining, building, repairing

        self.slowness = slowness #Used to slow unit down - 1 is move every logic cycle
        
        self.reload = 0 #If higher than weapon slowness, unit can fire
        self.move_charge = 0 #Unit can move only when it has enough of charge to overcome the slowness
        self.rotation = 0 #Stores rotation of unit
        self.mined = False #Unit can mine only once in logic cycle

        self.cargo = defaultdict(lambda: 0)
        self.cargo_capacity = cargo_capacity
        self.cargo_filled = 0 #How much cargo is already in

        #Graphic effects
        self.attack_laser = [] #Stores target for attack laser that is drawn when entity attacks

        self.sendDataToManager()

    def sendDataToManager(self): #Updatne premenne managera
        self.parent.all_units_grid_positions[self] = (self.x,self.y)

    def selfdraw(self):
        #Draws effects
        if self.attack_laser:
            self.parent.drawing_engine.draw_team_laser(self.x,self.y,self.attack_laser[0][0],self.attack_laser[0][1],self.parent.parent.team_number)
            self.attack_laser.pop()
        
        self.parent.drawing_engine.draw_team_object(self.name,self.x,self.y,self.parent.parent.team_number,self.rotation)

    def update(self): #Updates variables of unit every logic cycle
        self.move_charge += 1
        self.reload += 1

        self.mined = False
        self.worked_on_building = False

    def move(self,direction): #Function for moving, direction in form (+x,+y)
        if self.move_charge >= self.slowness: #Unit can move only one time every cycle
            self.move_charge = 0
            if not ((self.x+direction[0] >= len(self.parent.mapa) or self.x+direction[0] <= -1) or #Checks if the move doesn't lead out of the map
                    (self.y+direction[1] >= len(self.parent.mapa[0]) or self.y+direction[1] <= -1)):
                if self.parent.mapa[self.x+direction[0]][self.y+direction[1]]: #Checks if the tile terrain is passable                
                    if not ((self.x+direction[0],self.y+direction[1]) in self.parent.all_units_grid_positions.values() or #Checks if there isn't entity blocking tile
                            (self.x+direction[0],self.y+direction[1]) in self.parent.parent.blocked_positions.values()):
                        self.moved = True
                        #Moving itself
                        self.x += direction[0]
                        self.y += direction[1]

                        #Rotates unit
                        if direction[1] == -1: #Up
                            self.rotation = 0
                        elif direction[0] == -1: #Left
                            self.rotation = 90
                        elif direction[1] == 1: #Down
                            self.rotation = 180
                        elif direction[0] == 1: #Right
                            self.rotation = 270

                        self.sendDataToManager()

    def fire(self,position): #Uses weapon of unit to fire - position is in format (x,y)
        if self.reload >= self.weapon[2]:            
            self.reload = 0
            
            #Checks if the hit position is in range using Manhattan distance
            if self.weapon[1] < (abs(position[0]-self.x)+abs(position[1]-self.y)):
                return #The unit fired too far and wasted the shot that way

            #Graphic effect
            self.attack_laser = [position] #The longer the list is the longer it is drawn (for more frames)

            #Rotates unit when shooting - calculates rotation
            if abs(self.y-position[1]) > abs(self.x-position[0]):
                if self.y-position[1] < 0:
                    self.rotation = 180
                else:
                    self.rotation = 0
                    
            elif abs(self.y-position[1]) < abs(self.x-position[0]) or abs(self.y-position[1]) == abs(self.x-position[0]):
                if self.x-position[0] < 0:
                    self.rotation = 270
                else:
                    self.rotation = 90
                
            
            #Searches for entity located at that position and then damages it
            for entity, entity_position in self.parent.parent.blocked_positions.items():
                if entity_position == position:
                    entity.take_damage(self.weapon[0])

    def take_damage(self,damage): #Unit gets damaged from attack
        self.hp -= damage

    def build(self,position,target_object,recipe_number): #@todo add support for building units
        if (abs(self.x-position[0])+abs(self.y-position[1])) != 1: #Units can build only on tiles next to them
            return

        if ((position[0] >= len(self.parent.mapa) or position[0] <= -1) or #Checks if the target won't be built outside of the map
            (position[1] >= len(self.parent.mapa[0]) or position[1] <= -1)):
            return

        if not self.parent.mapa[position[0]][position[1]]: #Checks if the tile terrain is useable for construction (it must be passable)
            return

        if ((position[0],position[1]) in self.parent.all_units_grid_positions.values() or #Checks if there isn't entity blocking tile
            (position[0],position[1]) in self.parent.parent.blocked_positions):
            return
        
        requirements = self.parent.parent.recipe_manager.recipes[target_object][recipe_number] #Obtains requirements for the target object
        if not self.tool[1] in requirements[0]: #Checks if the unit has required building tool
            return

        for i in range(len(requirements[1])): #Iterates through resources required for construction
            if (requirements[2][i] and
                not self.parent.parent.resourceManager.resources[requirements[1][i]] >= int(requirements[2][i])): #Checks if the team has enough resources
                return #If not, cancels the building order

        #Remove used amount of resources from resource manager
        for i in range(len(requirements[1])): #Iterates through resources required for construction
            if requirements[2][i]:
                self.parent.parent.resourceManager.resources[requirements[1][i]] -= int(requirements[2][i])
                
        self.parent.parent.create_building(position[0],position[1],target_object,requirements[3],self.tool[1])
        #Requirements have been fulfiled and the building is being built (it has to be worked on still)

    def mine(self,position): #Mines from given position if possible
        if self.mined: #Unit has already mined during this logic cycle
            return

        #Graphic effect
        self.attack_laser = [position,"duration","duration"] #The longer the list is the longer it is drawn (for more frames)

        self.mined = True

        try:
            ore_name, ore_amount = self.parent.parent.parent.ore_manager.ores[(position[0],position[1])]
        except KeyError: #The ore has been mined out
            return

        if not self.tool[0] in self.parent.parent.parent.ore_manager.ore_types[ore_name]: #Unit doesn't have usable tool for mining the ore
            return

        mined_amount = int(math.ceil(ore_amount/10.0)) #Mines 10% of ore rounding up, that means the richer the ore is, the faster it is mined
        
        if self.cargo_capacity - self.cargo_filled < mined_amount: #If there is less cargo capacity free than should be mined out, the obtained ore amount is reduced
            mined_amount = self.cargo_capacity - self.cargo_filled

        #Loads ore into cargo
        self.cargo[ore_name] = self.cargo[ore_name] + mined_amount
        self.cargo_filled += mined_amount

        #Removes mined out ore amount from deposit
        self.parent.parent.parent.ore_manager.ores[(position[0],position[1])] = (ore_name,ore_amount-mined_amount)

    def unload_cargo(self,position,cargo_name,cargo_amount): #Unloads cargo to unit or building of the same team
        #Checks if the unload point is close enough
        if not (abs(position[0]-self.x)+abs(position[1]-self.y)) <= 1:
            return
        
        if (position[0],position[1]) in self.parent.all_units_grid_positions.values(): #Searches for units
            for unit, unit_position in self.parent.all_units_grid_positions.items():
                if (position[0],position[1]) == unit_position:
                    target = unit                
        elif (position[0],position[1]) in self.parent.parent.buildingManager.all_buildings_positions.values(): #Searches for buildings
            for building, building_position in self.parent.parent.buildingManager.all_buildings_positions.items():
                if (position[0],position[1]) == building_position:
                    target = building
        else: #Nothing was found
            return

        #Decreases offloaded cargo amount if there isn't enough space in target or there isn't enough of cargo
        if self.cargo[cargo_name] < cargo_amount:
            cargo_amount = self.cargo[cargo_name]

        cargo_amount = min([cargo_amount,target.cargo_capacity - target.cargo_filled])

        target.cargo[cargo_name] = target.cargo[cargo_name] + cargo_amount #Puts cargo into target
        target.cargo_filled += cargo_amount
        
        self.cargo[cargo_name] = self.cargo[cargo_name] - cargo_amount #Removes cargo from here
        self.cargo_filled -= cargo_amount

    def work_on_building(self,position): #Works on building in construction
        if self.worked_on_building:
            return

        #Graphic effect
        self.attack_laser = [position,"duration","duration"] #The longer the list is the longer it is drawn (for more frames)

        self.worked_on_building = True

        if not position in self.parent.parent.buildingManager.all_buildings_positions.values():
            return

        for building, building_position in self.parent.parent.buildingManager.all_buildings_positions.items():
            if building_position == position:
                if building.tool_to_build == self.tool[1]:
                    building.time_to_finish -= 1
                return


class SimpleUnitManager():
    def __init__(self, parent, mapa, drawing_engine):
        self.parent = parent
        self.mapa = mapa
        
        self.all_units = []
        self.all_units_grid_positions = {}

        self.drawing_engine = drawing_engine

        #Identifier system used for commanding the units
        self.id_system = {}
        self.current_identifier_number = 0

    def create_unit(self, x, y, unit_type):
        hp, weapon_type, tool_type, cargo_capacity, slowness = self.parent.UBWTTypes.unit_types[unit_type]
        weapon = self.parent.UBWTTypes.weapon_types[weapon_type]
        tool = self.parent.UBWTTypes.tool_types[tool_type]
        new_unit = BasicUnit(self, unit_type, x, y, hp, weapon, tool, cargo_capacity, slowness)
        self.all_units.append(new_unit)

        #Adds unit into ID system
        self.id_system[self.current_identifier_number] = new_unit
        new_unit.id = self.current_identifier_number
        self.current_identifier_number += 1

    def draw_all_units(self):
        for i in self.all_units:
            i.selfdraw()

    def detect_unit_using_position(self,x,y):
        for unit in self.all_units:
            if unit.x == x and unit.y == y:
                return unit
        return False
            
    def update(self):
        #First removes all destroyed units
        for i in range(len(self.all_units)-1,-1,-1): #Iterates from back to front so destroyed units can be removed
            if not self.all_units[i].hp > 0:
                #Removes all references to the unit
                del self.all_units_grid_positions[self.all_units[i]]
                del self.id_system[self.all_units[i].id]
                self.all_units.pop(i)
        for i in range(len(self.all_units)):
            self.all_units[i].update()

        return self.all_units_grid_positions
