#Server file
#Imports
import moduleNetwork
import simple_modules.moduleFilePathFormer #Creates paths to files #@todo replace with something better
import moduleMapLoader
import moduleTeamsManager

debug_mode = False #Removes user input when True

#Pre-game configuration
if debug_mode:
    client_count = 1 #Identical with team count unless there should be uncontrollable units and buildings
    team_count = 5
    map_path = "Example_map" #Path to map folder from the folder Maps in Game_folder, folders on the path should be separated by :
else:
    try:
        client_count = int(input("Enter how many clients will join (positive integer): "))
        team_count = int(input("Enter how many teams will be in game (positive integer, not smaller than client count): "))
        map_path = input("Enter path to map folder from folder Maps, use separator : instead of  OS separators: ")
    except TypeError:
        print("Client count or team count input is not integer")
        input("Press ENTER to end the program")
        raise

#Server socket initialization
port = 8192
server_socket = moduleNetwork.ServerSocket(port)

#FilePathFormer initialization #@todo replace with something better
path_creator = simple_modules.moduleFilePathFormer.FilePathFormer("Game_folder")

#Loads map files - necessary for game simulation
true_map_path = path_creator.create_path_to_file(["Maps"]+map_path.split(":")) #Finalizes the true map path so it will be usable
terrain_map, unit_file, building_file, ore_file = moduleMapLoader.load_map(true_map_path) #Uses the path to load the files
#unit_file, building_file and ore_file have to be run through the map loader again with teams_manager because they are used to write data into it

#Creates TeamsManager, to manage nearly everything #@todo rename to GameManager maybe?
teams_manager = moduleTeamsManager.TeamsManager(team_count,terrain_map,"")

#Writes data from unit_file, building_file and ore_file into teams_manager
moduleMapLoader.finalize_loading(unit_file,building_file,ore_file,teams_manager)

print("Server has been started with name "+str(moduleNetwork.get_adress())+" and IPv4 adress "+str(moduleNetwork.get_ip_adress())+" on port "+str(port))

client_number = 0 #Every client gets its number so it knows to which team it belongs
team_to_connection = {} #Dictionary storing connection:team_number pairs
#Accepts connections from clients
for i in range(client_count):
    print("Waiting for "+str(client_count-client_number)+" clients/client")
    client = server_socket.accept() #Server accepts new connection
    
    #Server sends team number to client
    moduleNetwork.send(client,str(client_number))
    team_to_connection[client_number] = client
    client_number += 1
    
    #Server sends total number of teams to client
    moduleNetwork.send(client,str(team_count))
    
    #Server sends path to the map folder so client can load the map
    moduleNetwork.send(client,map_path)

cycle = True

dropped_teams = [] #Stores defeated teams numbers so filler orders can be used when team disconnects

print("Game has started")

while cycle == True:
    #Receives input from clients
    client_inputs = server_socket.receive_from_all()

    for team_number in dropped_teams:
        client_inputs[team_number] = ""

    for team_number in range(len(client_inputs)): #Checks if some client wants to disconnect
        if client_inputs[team_number] == "disconnect":

            client_inputs[team_number] = "" #Fills the order place for the dropped team
            
            print("Team "+str(team_number)+" has been disconnected")

            #Closes the socket
            server_socket.connections[team_number].close()
            
            #Removes dropped team client socket from the server socket
            server_socket.adresses[team_number] = ""
            server_socket.connections[team_number] = ""
            
            dropped_teams.append(team_number)

    #Simulates the game
    teams_manager.give_orders_to_teams(client_inputs) #Gives the received controllers input to team managers
    teams_manager.update() #Everything updates

    #Checks which teams have been defeated (team has to lose all units and buildings to be defeated)
    for team_number in team_to_connection.keys():
        if (not team_number in dropped_teams and
            not teams_manager.teamManagers[team_number].unitManager.all_units and not teams_manager.teamManagers[team_number].buildingManager.all_buildings):
            print("Team "+str(team_number)+" has been defeated")
            moduleNetwork.send(team_to_connection[team_number],"You have been defeated")

            #Closes the socket
            server_socket.connections[team_number].close()

            #Removes dropped team client socket from the server socket
            server_socket.adresses[team_number] = ""
            server_socket.connections[team_number] = ""
            
            dropped_teams.append(team_number)
            
    
    message_for_clients = ""
    for i in client_inputs:
        message_for_clients += i + "?:?" #?:? is separator of the input messages

    server_socket.send_everyone(message_for_clients) #Server sends every client all the received messages

    if len(dropped_teams) == client_count: #Shutdowns the server if every client disconnects
        cycle = False
        print("Every client has disconnected, server shutting down")
        server_socket.socket.close()
        input("Press ENTER to end the program")

